from typing import Optional, List, Union, Literal
from loguru import logger

from pysus.ftp import Database, Directory, File
from pysus.ftp.utils import zfill_year, to_list


class IBGEDATASUS(Database):
    name = "IBGE-DataSUS"
    paths = (
        Directory("/dissemin/publicos/IBGE/POP"),
        Directory("/dissemin/publicos/IBGE/censo"),
        Directory("/dissemin/publicos/IBGE/POPTCU"),
        Directory("/dissemin/publicos/IBGE/projpop"),
        # Directory("/dissemin/publicos/IBGE/Auxiliar") # this has a different file name pattern
    )
    metadata = {
        "long_name": "Populaçao Residente, Censos, Contagens "
                     "Populacionais e Projeçoes Intercensitarias",
        "source": "ftp://ftp.datasus.gov.br/dissemin/publicos/IBGE",
        "description": (
            "São aqui apresentados informações sobre a população residente, "
            "estratificadas por município, faixas etárias e sexo, obtidas a "
            "partir dos Censos Demográficos, Contagens Populacionais "
            "e Projeções Intercensitárias."
        ),
    }

    def describe(self, file: File) -> dict:
        if file.extension.upper() in [".ZIP"]:
            year = file.name.split('.')[0][-2:]
            description = {
                "name": str(file.basename),
                "year": zfill_year(year),
                "size": file.info["size"],
                "last_update": file.info["modify"]
            }
            return description
        elif file.extension.upper() == ".DBF":
            year = file.name[-2:]
            description = {
                "name": str(file.basename),
                "year": zfill_year(year),
                "size": file.info["size"],
                "last_update": file.info["modify"]
            }
            return description
        return {}

    def format(self, file: File) -> tuple:
        return file.name[-2:],

    def get_files(
        self,
        source: Literal["POP", "censo", "POPTCU", "projpop"] = "POPTCU",
        year: Optional[Union[str, int, list]] = None,
        *args, **kwargs
    ) -> List[File]:
        sources = ["POP", "censo", "POPTCU", "projpop"]
        source_dir = None

        for dir in self.paths:
            if source in sources and source in dir.path:
                source_dir = dir

        if not source_dir:
            raise ValueError(f"Unkown source {source}. Options: {sources}")

        files = source_dir.content

        if year:
            if isinstance(year, (str, int)):
                files = [
                    f for f in files if
                    self.describe(f)["year"] == zfill_year(year)
                ]
            elif isinstance(year, list):
                files = [
                    f for f in files
                    if str(self.describe(f)["year"])
                    in [str(zfill_year(y)) for y in year]
                ]

        return files



"""
Downloads SIA data from Datasus FTP server
Created on 21/09/18
by fccoelho
Modified on 22/11/22
by bcbernardo
license: GPL V3 or Later
"""
from pprint import pprint
from typing import Dict, Tuple, Union

from loguru import logger

from pysus.ftp import CACHEPATH
from pysus.ftp.databases.sia import SIA
from pysus.ftp.utils import parse_UFs

sia = SIA().load()


group_dict: Dict[str, Tuple[str, int, int]] = {
    "PA": ("Produção Ambulatorial", 7, 1994),
    "BI": ("Boletim de Produção Ambulatorial individualizado", 1, 2008),
    "AD": ("APAC de Laudos Diversos", 1, 2008),
    "AM": ("APAC de Medicamentos", 1, 2008),
    "AN": ("APAC de Nefrologia", 1, 2008),
    "AQ": ("APAC de Quimioterapia", 1, 2008),
    "AR": ("APAC de Radioterapia", 1, 2008),
    "AB": ("APAC de Cirurgia Bariátrica", 1, 2008),
    "ACF": ("APAC de Confecção de Fístula", 1, 2008),
    "ATD": ("APAC de Tratamento Dialítico", 1, 2008),
    "AMP": ("APAC de Acompanhamento Multiprofissional", 1, 2008),
    "SAD": ("RAAS de Atenção Domiciliar", 1, 2008),
    "PS": ("RAAS Psicossocial", 1, 2008),
}


def get_available_years(
    group: str,
    states: Union[str, list] = None,
):
    """
    Get SIA years for group and/or state and returns a list of years
    :param group:
        PA: Produção Ambulatorial (7, 1994)
        BI: Boletim de Produção Ambulatorial individualizado (1, 2008)
        AD: APAC de Laudos Diversos (1, 2008)
        AM: APAC de Medicamentos (1, 2008)
        AN: APAC de Nefrologia (1, 2008)
        AQ: APAC de Quimioterapia (1, 2008)
        AR: APAC de Radioterapia (1, 2008)
        AB: APAC de Cirurgia Bariátrica (1, 2008)
        ACF: APAC de Confecção de Fístula (1, 2008)
        ATD: APAC de Tratamento Dialítico (1, 2008)
        AMP: APAC de Acompanhamento Multiprofissional (1, 2008)
        SAD: RAAS de Atenção Domiciliar (1, 2008)
        PS: RAAS Psicossocial (1, 2008)
    :param states: 2 letter state code, can be a list of UFs
    """
    ufs = parse_UFs(states)

    years = dict()
    for uf in ufs:
        files = sia.get_files(group, uf=uf)
        years[uf] = set(sorted([sia.describe(f)["year"] for f in files]))

    if len(set([len(v) for v in years.values()])) > 1:
        logger.warning(f"Distinct years were found for UFs: {years}")

    return sorted(list(set.intersection(*map(set, years.values()))))


def show_datatypes():
    pprint(group_dict)


def download(
    states: Union[str, list],
    years: Union[str, list, int],
    months: Union[str, list, int],
    groups: Union[str, list],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Download SIASUS records for state year and month and returns dataframe
    :param states: 2 letter state code, can be a list
    :param years: 4 digit integer, can be a list
    :param months: 1 to 12, can be a list
    :param data_dir: whether to cache files locally. default is True
    :param group: SIA groups. For all groups, refer to `sia.groups`
    :return: list of downloaded ParquetData
    """
    files = sia.get_files(
        group=groups, uf=states, year=years, month=months
    )
    return sia.download(files, local_dir=data_dir)



# -*- coding:utf-8 -*-
u"""
Created on 2023/12/12
by luabida
license: GPL V3 or Later
"""
import unittest
import datetime
from pathlib import Path

import pandas as pd

from pysus.ftp import File, CACHEPATH
from pysus.data.local import ParquetSet


class TestFile(unittest.TestCase):

    def setUp(self):
        path = "/dissemin/publicos/SIM/CID10/DORES/"
        name = "DOAC1996.dbc"
        info = {
            "size": 76107,
            "type": "file",
            "modify": datetime.datetime(2020, 1, 31, 14, 48)
        }

        self.file = File(path, name, info)

    def test_file_initialization(self):
        file = self.file

        expected_path = "/dissemin/publicos/SIM/CID10/DORES/DOAC1996.dbc"
        self.assertEqual(file.path, expected_path)

        self.assertEqual(file.name, "DOAC1996")

        self.assertEqual(file.extension, ".dbc")

        self.assertEqual(file.basename, "DOAC1996.dbc")

        expected_info = {
            'size': '76.1 kB',
            'type': 'DBC file',
            'modify': '2020-01-31 02:48PM'
        }
        self.assertEqual(file.info, expected_info)

    def test_file_download(self):
        parquet = self.file.download()

        self.assertIsInstance(parquet, ParquetSet)

        self.assertTrue("size" in parquet.info)

        local_cache = Path(CACHEPATH)
        expected_local_path = local_cache / "DOAC1996.parquet"
        self.assertTrue(expected_local_path.exists())
        self.assertEqual(Path(str(parquet)), expected_local_path)

        df = parquet.to_dataframe()
        self.assertIsInstance(df, pd.DataFrame)
        self.assertFalse(df.empty)



from typing import Optional, List, Union

from pysus.ftp import Database, Directory, File
from pysus.ftp.utils import zfill_year, to_list


class SINAN(Database):
    name = "SINAN"
    paths = (
        Directory("/dissemin/publicos/SINAN/DADOS/FINAIS"),
        Directory("/dissemin/publicos/SINAN/DADOS/PRELIM"),
    )
    metadata = {
        "long_name": "Doenças e Agravos de Notificação",
        "source": "https://portalsinan.saude.gov.br/",
        "description": (
            "The Notifiable Diseases Information System - Sinan is primarily"
            "fed by the notification and investigation of cases of diseases "
            "and conditions listed in the national list of compulsorily notifiable "
            "diseases (Consolidation Ordinance No. 4, September 28, 2017, Annex)."
            "However, states and municipalities are allowed to include other "
            "important health problems in their region, such as difilobotriasis "
            "in the municipality of São Paulo. Its effective use enables the dynamic "
            "diagnosis of the occurrence of an event in the population, providing "
            "evidence for causal explanations of compulsorily notifiable diseases "
            "and indicating risks to which people are exposed. This contributes to "
            "identifying the epidemiological reality of a specific geographical area. "
            "Its systematic, decentralized use contributes to the democratization of "
            "information, allowing all healthcare professionals to access and make "
            "it available to the community. Therefore, it is a relevant tool to assist"
            " in health planning, define intervention priorities, and evaluate the "
            "impact of interventions."
        ),
    }

    diseases = {
        "ACBI": "Acidente de trabalho com material biológico",
        "ACGR": "Acidente de trabalho",
        "ANIM": "Acidente por Animais Peçonhentos",
        "ANTR": "Atendimento Antirrabico",
        "BOTU": "Botulismo",
        "CANC": "Cancêr relacionado ao trabalho",
        "CHAG": "Doença de Chagas Aguda",
        "CHIK": "Febre de Chikungunya",
        "COLE": "Cólera",
        "COQU": "Coqueluche",
        "DENG": "Dengue",
        "DERM": "Dermatoses ocupacionais",
        "DIFT": "Difteria",
        "ESQU": "Esquistossomose",
        "EXAN": "Doença exantemáticas",
        "FMAC": "Febre Maculosa",
        "FTIF": "Febre Tifóide",
        "HANS": "Hanseníase",
        "HANT": "Hantavirose",
        "HEPA": "Hepatites Virais",
        "IEXO": "Intoxicação Exógena",
        "INFL": "Influenza Pandêmica",
        "LEIV": "Leishmaniose Visceral",
        "LEPT": "Leptospirose",
        "LERD": "LER/Dort",
        "LTAN": "Leishmaniose Tegumentar Americana",
        "MALA": "Malária",
        "MENI": "Meningite",
        "MENT": "Transtornos mentais relacionados ao trabalho",
        "NTRA": "Notificação de Tracoma",
        "PAIR": "Perda auditiva por ruído relacionado ao trabalho",
        "PEST": "Peste",
        "PFAN": "Paralisia Flácida Aguda",
        "PNEU": "Pneumoconioses realacionadas ao trabalho",
        "RAIV": "Raiva",
        "SDTA": "Surto Doenças Transmitidas por Alimentos",
        "SIFA": "Sífilis Adquirida",
        "SIFC": "Sífilis Congênita",
        "SIFG": "Sífilis em Gestante",
        "SRC": "Síndrome da Rubéola Congênia",
        "TETA": "Tétano Acidental",
        "TETN": "Tétano Neonatal",
        "TOXC": "Toxoplasmose Congênita",
        "TOXG": "Toxoplasmose Gestacional",
        "TRAC": "Inquérito de Tracoma",
        "TUBE": "Tuberculose",
        "VARC": "Varicela",
        "VIOL": "Violência doméstica, sexual e/ou outras violências",
        "ZIKA": "Zika Vírus",
    }

    def describe(self, file: File) -> dict:
        if file.extension.upper() == ".DBC":
            dis_code, year = self.format(file)

            description = {
                "name": str(file.basename),
                "disease": self.diseases[dis_code],
                "year": zfill_year(year),
                "size": file.info["size"],
                "last_update": file.info["modify"]
            }
            return description
        return {}

    def format(self, file: File) -> tuple:
        year = file.name[-2:]

        if file.name.startswith("SRC"):
            dis_code = file.name[:3]
        elif file.name == "LEIBR22":
            dis_code = "LEIV"  # MISPELLED FILE NAME
        elif file.name == "LERBR19":
            dis_code = "LERD"  # ANOTHER ONE
        else:
            dis_code = file.name[:4]

        return dis_code, zfill_year(year)

    def get_files(
        self,
        dis_code: Optional[Union[str, list]] = None,
        year: Optional[Union[str, int, list]] = None,
    ) -> List[File]:
        files = list(filter(
            lambda f: f.extension.upper() in [".DBC", ".DBF"], self.files
        ))

        if dis_code:
            codes = [c.upper() for c in to_list(dis_code)]

            if codes and not all(code in self.diseases for code in codes):
                raise ValueError(
                    "Unknown disease(s): "
                    f"{set(codes).difference(set(self.diseases))}"
                )

            files = list(filter(lambda f: self.format(f)[0] in codes, files))

        if year or str(year) in ["0", "00"]:
            years = (
                [zfill_year(str(y)[-2:]) for y in to_list(year)]
            )
            files = list(filter(lambda f: self.format(f)[1] in years, files))

        return files



"""
Downloads data made available by the Infogripe service
"""

import pandas as pd

BASEURL = r'https://gitlab.fiocruz.br/marcelo.gomes/infogripe/-/raw/master/Dados/InfoGripe/'
DATASETS = {
    'Alerta de situação': r'tabela_de_alerta.csv',
    'Casos por idade, sexo e virus': r'dados_semanais_faixa_etaria_sexo_virus.csv.gz',
    'Casos Totais e estimativas': r'serie_temporal_com_estimativas_recentes.csv.gz',
    'Valores esperados por localidades': 'valores_esperados_por_localidade.csv',
}


def list_datasets():
    return list(DATASETS.keys())


def download(dataset_name):
    url = BASEURL + DATASETS[dataset_name]
    df = pd.read_csv(url, delimiter=';', decimal=',')
    return df



from typing import Union

from loguru import logger

from pysus.ftp.databases.cnes import CNES
from pysus.ftp import CACHEPATH
from pysus.ftp.utils import parse_UFs


cnes = CNES().load()


group_dict = {
    'LT': ['Leitos - A partir de Out/2005', 10, 2005],
    'ST': ['Estabelecimentos - A partir de Ago/2005', 8, 2005],
    'DC': ['Dados Complementares - A partir de Ago/2005', 8, 2005],
    'EQ': ['Equipamentos - A partir de Ago/2005', 8, 2005],
    'SR': ['Serviço Especializado - A partir de Ago/2005', 8, 2005],
    'HB': ['Habilitação - A partir de Mar/2007', 3, 2007],
    'PF': ['Profissional - A partir de Ago/2005', 8, 2005],
    'EP': ['Equipes - A partir de Abr/2007', 5, 2007],
    'IN': ['Incentivos - A partir de Nov/2007', 11, 2007],
    'RC': ['Regra Contratual - A partir de Mar/2007', 3, 2007],
    'EE': ['Estabelecimento de Ensino - A partir de Mar/2007', 3, 2007],
    'EF': ['Estabelecimento Filantrópico - A partir de Mar/2007', 3, 2007],
    'GM': ['Gestão e Metas - A partir de Jun/2007', 6, 2007],
}


def get_available_years(
    group: str,
    states: Union[str, list] = None,
):
    """
    Get CNES years for group and/or state and returns a
    list of years
    :param group:
        LT – Leitos - A partir de Out/2005
        ST – Estabelecimentos - A partir de Ago/2005
        DC - Dados Complementares - A partir de Ago/2005
        EQ – Equipamentos - A partir de Ago/2005
        SR - Serviço Especializado - A partir de Ago/2005
        HB – Habilitação - A partir de Mar/2007
        PF – Profissional - A partir de Ago/2005
        EP – Equipes - A partir de Abr/2007
        IN – Incentivos - A partir de Nov/2007
        RC - Regra Contratual - A partir de Mar/2007
        EE - Estabelecimento de Ensino - A partir de Mar/2007
        EF - Estabelecimento Filantrópico - A partir de Mar/2007
        GM - Gestão e Metas - A partir de Jun/2007
    :param states: 2 letter state code, can be a list of UFs
    """
    cnes.load(group)

    ufs = parse_UFs(states)

    years = dict()
    for uf in ufs:
        files = cnes.get_files(group, uf=uf)
        years[uf] = sorted([cnes.describe(f)["year"] for f in files])

    if len(set([len(v) for v in years.values()])) > 1:
        logger.warning(f"Distinct years were found for UFs: {years}")

    return sorted(list(set.intersection(*map(set, years.values()))))


def download(
    group: str,
    states: Union[str, list],
    years: Union[str, list, int],
    months: Union[str, list, int],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Download CNES records for group, state, year and month and returns a
    list of local parquet files
    :param group:
        LT – Leitos - A partir de Out/2005
        ST – Estabelecimentos - A partir de Ago/2005
        DC - Dados Complementares - A partir de Ago/2005
        EQ – Equipamentos - A partir de Ago/2005
        SR - Serviço Especializado - A partir de Ago/2005
        HB – Habilitação - A partir de Mar/2007
        PF – Profissional - A partir de Ago/2005
        EP – Equipes - A partir de Abr/2007
        IN – Incentivos - A partir de Nov/2007
        RC - Regra Contratual - A partir de Mar/2007
        EE - Estabelecimento de Ensino - A partir de Mar/2007
        EF - Estabelecimento Filantrópico - A partir de Mar/2007
        GM - Gestão e Metas - A partir de Jun/2007
    :param months: 1 to 12, can be a list of years
    :param states: 2 letter state code, can be a list of UFs
    :param years: 4 digit integer, can be a list of years
    """
    files = cnes.get_files(group, states, years, months)
    return cnes.download(files, local_dir=data_dir)



import os
import struct
import logging
from datetime import datetime
from pathlib import Path

import pandas as pd
import pyarrow as pa
import pyarrow.parquet as pq
from dbfread import DBF
from pyreaddbc import dbc2dbf


def dbc_to_dbf(dbc: str, _pbar=None) -> str:
    """
    Parses DBC files into DBFs
    """
    path = Path(dbc)

    if path.suffix.lower() != ".dbc":
        raise ValueError(f"Not a DBC file: {path}")

    dbf = path.with_suffix(".dbf")

    if _pbar:
        _pbar.reset(total=1)
        _pbar.set_description(f"{dbf.name}")

    _parquet = path.with_suffix(".parquet")
    if _parquet.exists():
        path.unlink(missing_ok=True)
        dbf.unlink(missing_ok=True)
        return str(_parquet)

    if dbf.exists():
        path.unlink(missing_ok=True)
        return str(dbf)

    dbc2dbf(str(path), str(dbf))
    path.unlink()

    if _pbar:
        _pbar.update(1)

    return str(dbf)


def stream_dbf(dbf, chunk_size=30000):
    """Fetches records in parquet chunks to preserve memory"""
    data = []
    i = 0
    for records in dbf:
        data.append(records)
        i += 1
        if i == chunk_size:
            yield data
            data = []
            i = 0
    else:
        yield data


def decode_column(value):
    """
    Decodes binary data to str
    """
    if isinstance(value, bytes):
        return value.decode(encoding="iso-8859-1").replace("\x00", "")

    if isinstance(value, str):
        return str(value).replace("\x00", "")

    return value


def dbf_to_parquet(dbf: str, _pbar=None) -> str:
    """
    Parses DBF file into parquet to preserve memory
    """
    path = Path(dbf)

    if path.suffix.lower() != ".dbf":
        raise ValueError(f"Not a DBF file: {path}")

    parquet = path.with_suffix(".parquet")

    approx_final_size = (
        os.path.getsize(path) / 200
    )  # TODO: not best approx size
    if _pbar:
        _pbar.unit = "B"
        _pbar.unit_scale = True
        _pbar.reset(total=approx_final_size)
        _pbar.set_description(f"{parquet.name}")

    if parquet.exists():
        if _pbar:
            _pbar.update(approx_final_size - _pbar.n)
        return str(parquet)

    parquet.absolute().mkdir()

    try:
        chunk_size = 30_000
        for chunk in stream_dbf(
            DBF(path, encoding="iso-8859-1", raw=True), chunk_size
        ):
            if _pbar:
                _pbar.update(chunk_size)

            chunk_df = pd.DataFrame(chunk)
            table = pa.Table.from_pandas(chunk_df.map(decode_column))
            pq.write_to_dataset(table, root_path=str(parquet))
    except struct.error as err:
        if _pbar:
            _pbar.close()
        Path(path).unlink()
        parquet.rmdir()
        raise err

    if _pbar:
        _pbar.update(approx_final_size - _pbar.n)

    path.unlink()

    return str(parquet)


def parse_dftypes(df: pd.DataFrame) -> pd.DataFrame:
    """
    Parse DataFrame values, cleaning blank spaces if needed
    and converting dtypes into correct types.
    """

    def map_column_func(column_names: list[str], func):
        # Maps a function to each value in each column
        columns = [c for c in df.columns if c in column_names]
        df[columns] = df[columns].map(func)

    def str_to_int(string: str):
        # If removing spaces, all characters are int,
        # return int(value). @warning it removes in between
        # spaces as well
        if str(string).replace(" ", "").isnumeric():
            return int(string.replace(" ", ""))
        return string

    def str_to_date(string: str):
        if isinstance(string, str):
            try:
                return datetime.strptime(string, "%Y%m%d").date()
            except ValueError:
                # Ignore errors, bad value
                return string
        return string

    map_column_func(["DT_NOTIFIC", "DT_SIN_PRI"], str_to_date)
    map_column_func(["CODMUNRES", "SEXO"], str_to_int)

    df = df.map(
        lambda x: "" if str(x).isspace() else x
    )  # Remove all space values

    df = df.convert_dtypes()
    return df



# -*- coding:utf-8 -*-
u"""
Created on 2023/12/12
by luabida
license: GPL V3 or Later
"""
import unittest
import datetime

from unittest.mock import patch, MagicMock

from pysus.ftp.databases.ciha import CIHA
from pysus.ftp import File


class TestCIHADatabase(unittest.TestCase):

    def test_ciha(self):
        mock_content = {
            "CIHAAC1101.dbc": File(
                path="/dissemin/publicos/CIHA/201101_/Dados/CIHAAC1101.dbc",
                name="CIHAAC1101.dbc",
                info={
                    'size': 7803,
                    'type': 'file',
                    'modify': datetime.datetime(2023, 10, 6, 10, 17)
                }
            ),
            "CIHAAC1102.dbc": File(
                path="/dissemin/publicos/CIHA/201101_/Dados/CIHAAC1102.dbc",
                name="CIHAAC1102.dbc",
                info={
                    'size': 9959,
                    'type': 'file',
                    'modify': datetime.datetime(2023, 10, 6, 10, 17)
                }
            ),
            "CIHAAC1103.dbc": File(
                path="/dissemin/publicos/CIHA/201101_/Dados/CIHAAC1103.dbc",
                name="CIHAAC1103.dbc",
                info={
                    'size': 8308,
                    'type': 'file',
                    'modify': datetime.datetime(2023, 10, 6, 10, 17)
                }
            ),
        }

        with patch(
            'pysus.ftp.databases.ciha.CIHA',
            return_value=MagicMock(__content__=mock_content)
        ) as mock_ciha:
            ciha = CIHA()
            ciha.__content__ = mock_ciha().__content__

            descriptions = [ciha.describe(file) for file in ciha.files]
            expected_descriptions = [
                {'name': 'CIHAAC1101.dbc',
                 'group': 'Comunicação de Internação Hospitalar e Ambulatorial',
                 'uf': 'Acre',
                 'month': 'Janeiro',
                 'year': 2011,
                 'size': '7.8 kB',
                 'last_update': '2023-10-06 10:17AM'},
                {'name': 'CIHAAC1102.dbc',
                 'group': 'Comunicação de Internação Hospitalar e Ambulatorial',
                 'uf': 'Acre',
                 'month': 'Fevereiro',
                 'year': 2011,
                 'size': '10.0 kB',
                 'last_update': '2023-10-06 10:17AM'},
                {'name': 'CIHAAC1103.dbc',
                 'group': 'Comunicação de Internação Hospitalar e Ambulatorial',
                 'uf': 'Acre',
                 'month': 'Março',
                 'year': 2011,
                 'size': '8.3 kB',
                 'last_update': '2023-10-06 10:17AM'}
            ]

            self.assertEqual(descriptions, expected_descriptions)

            formats = [ciha.format(file) for file in ciha.files]
            expected_formats = [
                ('CIHA', 'AC', 2011, '01'),
                ('CIHA', 'AC', 2011, '02'),
                ('CIHA', 'AC', 2011, '03')
            ]
            self.assertEqual(formats, expected_formats)

            get_files = ciha.get_files(
                uf='AC', year=2011, month=1
            )
            self.assertEqual(get_files, [ciha.files[0]])



"""
Downloads SIH data from Datasus FTP server
Created on 21/09/18
by fccoelho
license: GPL V3 or Later
"""
from typing import Union

from loguru import logger

from pysus.ftp import CACHEPATH
from pysus.ftp.databases.sih import SIH
from pysus.ftp.utils import parse_UFs

sih = SIH().load()


def get_available_years(
    group: str,
    states: Union[str, list] = None,
) -> list:
    """
    Get SIH years for group and/or state and returns a list of years
    :param group:
        RD: AIH Reduzida
        RJ: AIH Rejeitada
        ER: AIH Rejeitada com erro
        SP: Serviços Profissionais
        CH: Cadastro Hospitalar
        CM: # TODO
    :param states: 2 letter uf code, can be a list. E.g: "SP" or ["SP", "RJ"]
    :return: list of available years
    """
    ufs = parse_UFs(states)

    years = dict()
    for uf in ufs:
        files = sih.get_files(group, uf=uf)
        years[uf] = set(sorted([sih.describe(f)["year"] for f in files]))

    if len(set([len(v) for v in years.values()])) > 1:
        logger.warning(f"Distinct years were found for UFs: {years}")

    return sorted(list(set.intersection(*map(set, years.values()))))


def download(
    states: Union[str, list],
    years: Union[str, list, int],
    months: Union[str, list, int],
    groups: Union[str, list],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Download SIH records for state, year and month
    :param states: 2 letter state code, can be a list
    :param years: 4 digit integer, can be a list
    :param months: 1 to 12, can be a list
    :param groups: the groups of datasets to be downloaded.
                   See `sih.groups`
    :param data_dir: Directory where parquets will be downloaded.
    :return: list with the downloaded files as ParquetData objects
    """
    files = sih.get_files(
        group=groups, uf=states, month=months, year=years
    )
    return sih.download(files, local_dir=data_dir)



import unittest

import pandas as pd
import pytest
from pysus.online_data.Infodengue import download, normalize, search_string


class InfoDengueTestCase(unittest.TestCase):
    @pytest.mark.timeout(5)
    def test_search_string(self):
        get_from_dict = search_string("Curitiba")
        cites_mathes = {
            "Acajutiba": 2900306,
            "Aratiba": 4300901,
            "Bacurituba": 2101350,
            "Buriti": 2102200,
            "Buriti Bravo": 2102309,
            "Buritirama": 2904753,
            "Buritirana": 2102358,
            "Buritis": 3109303,
            "Buritizal": 3508207,
            "Caatiba": 2904803,
            "Caraíbas": 2906899,
            "Carnaíba": 2603900,
            "Caturité": 2504355,
            "Craíbas": 2702355,
            "Criciúma": 4204608,
            "Cristais": 3120201,
            "Cristal": 4306056,
            "Cristina": 3120508,
            "Cromínia": 5206503,
            "Cruzília": 3120805,
            "Cuiabá": 5103403,
            "Cuitegi": 2505204,
            "Curimatá": 2203206,
            "Curitiba": 4106902,
            "Curitibanos": 4204806,
            "Curiúva": 4107009,
            "Custódia": 2605103,
            "Cutias": 1600212,
            "Duartina": 3514502,
            "Guaraíta": 5209291,
            "Guariba": 3518602,
            "Guaribas": 2204550,
            "Ibatiba": 3202454,
            "Ibicuitinga": 2305332,
            "Irituia": 1503507,
            "Itagibá": 2915205,
            "Itaituba": 1503606,
            "Itaiçaba": 2306207,
            "Itatiba": 3523404,
            "Itaíba": 2607505,
            "Itiúba": 2917003,
            "Jequitibá": 3135704,
            "Juquitiba": 3526209,
            "Marituba": 1504422,
            "Mauriti": 2308104,
            "Mucurici": 3203601,
            "Muribeca": 2804300,
            "Muritiba": 2922300,
            "Peritiba": 4212601,
            "Piritiba": 2924801,
            "Taquarituba": 3553807,
            "Tumiritinga": 3169505,
            "Turiúba": 3555208,
            "Umburatiba": 3170305,
            "Urucurituba": 1304401,
        }
        pattern_city_names = search_string(substr="r de jAiro")

        self.assertIsInstance(get_from_dict, dict)
        self.assertEqual(cites_mathes, get_from_dict)
        self.assertIn("Rio de Janeiro", pattern_city_names.keys())
        self.assertIn(4204806, get_from_dict.values())

    @pytest.mark.timeout(5)
    def test_normalize(self):
        normalized_str = normalize("Rio das Ostras")

        substr_list = normalized_str.split(".")

        self.assertIsInstance(substr_list, list)
        # self.assertEqual(substr_list, ['rio', 'das', 'ostras'])
        self.assertEqual(normalized_str, "rio das ostras")

    @pytest.mark.timeout(5)
    def test_download(self):
        df = download(
            "dengue",
            202129,
            202152,
            "Rio de Janeiro",
        )
        df_size = (29, 24)

        self.assertIsInstance(df, pd.DataFrame)
        self.assertGreater(len(df), 0)
        self.assertEqual(df_size, df.shape)


if __name__ == "__main__":
    unittest.main()



# -*- coding:utf-8 -*-
u"""
Created on 2023/12/11
by luabida
license: GPL V3 or Later
"""
import unittest
import datetime

from unittest.mock import patch, MagicMock

from pysus.ftp.databases.sih import SIH
from pysus.ftp import File


class TestSIHDatabase(unittest.TestCase):

    def test_sih(self):
        mock_content = {
            "CHBR1901.dbc": File(
                path="/dissemin/publicos/SIHSUS/200801_/Dados/CHBR1901.dbc",
                name="CHBR1901.dbc",
                info={
                    'size': 196476,
                    'type': 'file',
                    'modify': datetime.datetime(2020, 3, 10, 14, 43)
                }
            ),
            "CHBR1902.dbc": File(
                path="/dissemin/publicos/SIHSUS/200801_/Dados/CHBR1902.dbc",
                name="CHBR1902.dbc",
                info={
                    'size': 196287,
                    'type': 'file',
                    'modify': datetime.datetime(2020, 3, 10, 14, 43)
                }
            ),
            "CHBR1903.dbc": File(
                path="/dissemin/publicos/SIHSUS/200801_/Dados/CHBR1903.dbc",
                name="CHBR1903.dbc",
                info={
                    'size': 196081,
                    'type': 'file',
                    'modify': datetime.datetime(2020, 3, 10, 14, 43)
                }
            ),
        }

        with patch(
            'pysus.ftp.databases.sih.SIH',
            return_value=MagicMock(__content__=mock_content)
        ) as mock_sih:
            sih = SIH()
            sih.__content__ = mock_sih().__content__

            descriptions = [sih.describe(file) for file in sih.files]
            expected_descriptions = [
                {'name': 'CHBR1901.dbc',
                 'group': 'Cadastro Hospitalar',
                 'uf': 'Brasil',
                 'month': 'Janeiro',
                 'year': 2019,
                 'size': '196.5 kB',
                 'last_update': '2020-03-10 02:43PM'},
                {'name': 'CHBR1902.dbc',
                 'group': 'Cadastro Hospitalar',
                 'uf': 'Brasil',
                 'month': 'Fevereiro',
                 'year': 2019,
                 'size': '196.3 kB',
                 'last_update': '2020-03-10 02:43PM'},
                {'name': 'CHBR1903.dbc',
                 'group': 'Cadastro Hospitalar',
                 'uf': 'Brasil',
                 'month': 'Março',
                 'year': 2019,
                 'size': '196.1 kB',
                 'last_update': '2020-03-10 02:43PM'}
            ]

            self.assertEqual(descriptions, expected_descriptions)

            formats = [sih.format(file) for file in sih.files]
            expected_formats = [
                ('CH', 'BR', 2019, '01'),
                ('CH', 'BR', 2019, '02'),
                ('CH', 'BR', 2019, '03')
            ]
            self.assertEqual(formats, expected_formats)

            get_files = sih.get_files(
                group='CH', uf='BR', year=2019, month=1
            )
            self.assertEqual(get_files, [sih.files[0]])
